@@ -100,7 +100,7 @@ def truncation_correction(projections, extension_fraction=0.1):
 
 
 def read_and_preprocess_txrm(projections_file, downsample_factor=1, angular_factor=1, do_truncation_correction=True,
-                             extension_fraction=0.1, datatype=np.float32):
+                             extension_fraction=0.1, datatype=np.float32, keep_rows=None):
     '''Preprocess the raw images, i.e. remove shifts, transform into line integral domain.
     Projections are extended for truncation correction if needed.
 
@@ -118,16 +118,23 @@ def read_and_preprocess_txrm(projections_file, downsample_factor=1, angular_fact
     x_shifts = metadata['x-shifts'][::angular_factor]
     y_shifts = metadata['y-shifts'][::angular_factor]
     reference = metadata['reference']
+    keep_rows = keep_rows.astype(np.bool)
+    if keep_rows is not None:
+        projection_height = int(np.sum(keep_rows) / downsample_factor)
+    else:
+        projection_height = int(np.ceil(float(metadata['image_height']) / downsample_factor))
     print('Preprocessing projections')
     if do_truncation_correction:
         projections = np.zeros((int(np.ceil(num_images / angular_factor)),
-                                int(np.ceil(float(metadata['image_height']) / downsample_factor)),
+                                projection_height,
+                                #int(np.ceil(float(metadata['image_height']) / downsample_factor)),
                                 int(np.ceil(float(metadata['image_width']) / downsample_factor)) + 2 * int(
                                     np.ceil(float(metadata['image_width']) / downsample_factor) * extension_fraction)),
                                dtype=datatype)
     else:
         projections = np.zeros((int(np.ceil(num_images / angular_factor)),
-                                int(np.ceil(float(metadata['image_height']) / downsample_factor)),
+                                projection_height,
+                                #int(np.ceil(float(metadata['image_height']) / downsample_factor)),
                                 int(np.ceil(float(metadata['image_width']) / downsample_factor))),
                                dtype=datatype)
 
@@ -141,6 +148,8 @@ def read_and_preprocess_txrm(projections_file, downsample_factor=1, angular_fact
         projection = np.expand_dims(projection, 0)
         projection = divide_by_reference(projection, reference)
         projection = revert_shifts(projection, x_shift, y_shift)
+        if keep_rows is not None:
+            projection = projection[:, keep_rows, :]
         if downsample_factor != 1 or angular_factor != 1:
             projection = downsample(projection, downsample_factor, angular_factor)
         projection = negative_logarithm(projection)
@@ -152,13 +161,15 @@ def read_and_preprocess_txrm(projections_file, downsample_factor=1, angular_fact
 
 
 def read_single_projection(projections_file, downsample_factor=1, angular_factor=1, do_truncation_correction=True,
-                             extension_fraction=0.1, datatype=np.float32):
+                             extension_fraction=0.1, datatype=np.float32, keep_rows=None):
     metadata = read_metadata(str(projections_file))
     reference = metadata['reference']
-
+    keep_rows = keep_rows.astype(np.bool)
     for projection in read_txrm_iterable(str(projections_file)):
         projection = np.expand_dims(projection, 0)
         projection = divide_by_reference(projection, reference)
+        if keep_rows is not None:
+            projection = projection[:, keep_rows, :]
         if downsample_factor != 1 or angular_factor != 1:
             projection = downsample(projection, downsample_factor, angular_factor)
         projection = negative_logarithm(projection)
